//+------------------------------------------------------------------+
//|                                                          SAR.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//|                                                    Parabolic.mq4 |
//|                      Copyright  2012, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"
 
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Lime
#property indicator_color2 Silver
#property indicator_width2 2
//---- input parameters
extern int ma_per  = 4;
extern int ma_meth = 2;
extern int ma_prix = 4;

extern double    Step=0.02;
extern double    Maximum=0.2;
//---- buffers
double SarBuffer[];
double ma[];
//----
int    save_lastreverse;
bool   save_dirlong;
double save_start;
double save_last_high;
double save_last_low;
double save_ep;
double save_sar;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0,DRAW_ARROW);
   SetIndexArrow(0,159);
   SetIndexBuffer(0,SarBuffer);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,ma);

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| save values to fast calculations on next calls                   |
//+------------------------------------------------------------------+
void SaveLastReverse(int last,bool dir,double start,double low,double high,double ep,double sar)
  {
   save_lastreverse=last;
   save_dirlong=dir;
   save_start=start;
   save_last_low=low;
   save_last_high=high;
   save_ep=ep;
   save_sar=sar;
  }
//+------------------------------------------------------------------+
//| Parabolic Sell And Reverse system                                |
//+------------------------------------------------------------------+
int start()
  {
   static bool first=true;
   bool   dirlong;
   double start,last_high,last_low;
   double ep,sar,price_low,price_high,price;
   int    i,counted_bars=IndicatorCounted();
//---- nothing to calculate
   if(Bars<3) return(0);
//---- first calculation?
   if(counted_bars==0 || first)
     {
      //---- initial settings
      i=Bars-2;
      first=false;
      dirlong=true;
      start=Step;
      last_high=-10000000.0;
      last_low=10000000.0;
      save_lastreverse=0;
      sar=0;
      while(i>0)
        {
         save_lastreverse=i;
         price_low=Low[i];
         price_high=High[i];
         if(last_low>price_low)
            last_low=price_low;
         if(last_high<price_high)
            last_high=price_high;
         if(price_high>High[i+1] && price_low>Low[i+1])
            break;
         if(price_high<High[i+1] && price_low<Low[i+1])
           {
            dirlong=false;
            break;
           }
         i--;
        }
      //---- initial zero
      int k=i;
      while(k<Bars)
        {
         SarBuffer[k]=0.0;
         k++;
        }
      //---- check further
      if(dirlong) { SarBuffer[i]=Low[i+1]; ep=High[i]; }
      else        { SarBuffer[i]=High[i+1]; ep=Low[i]; }
      i--;
     }
    else
     {
      //---- restore values from previous call to avoid full recalculation
      i=save_lastreverse;
      start=save_start;
      dirlong=save_dirlong;
      last_high=save_last_high;
      last_low=save_last_low;
      ep=save_ep;
      sar=save_sar;
     }
//----
   while(i>=0)
     {
      ma[i] = iMA(NULL,0,ma_per,0,ma_meth,ma_prix,i);
     
      price_low=  ma[i];//  Low[i];
      price_high=   ma[i];//  High[i];
      price=SarBuffer[i+1];
      //--- check for reverse
      if(dirlong && price_low<price)
        {
         SaveLastReverse(i+1,true,start,price_low,last_high,ep,sar);
         start=Step; dirlong=false;
         ep=price_low;
         last_low=price_low;
         SarBuffer[i]=last_high;
         i--;
         continue;
        }
      if(!dirlong && price_high>price)
        {
         SaveLastReverse(i+1,false,start,last_low,price_high,ep,sar);
         start=Step; dirlong=true;
         ep=price_high;
         last_high=price_high;
         SarBuffer[i]=last_low;
         i--;
         continue;
        }
      //--- calculate current value
      sar=price+start*(ep-price);
      //---- check long direction
      if(dirlong)
        {
         if(ep<price_high && (start+Step)<=Maximum)
            start+=Step;
         if(price_high< ma[i+1] && i==Bars-2)
            sar=price;
         if(sar>ma[i+1]) sar=ma[i+1];
         if(sar>ma[i+2]) sar=ma[i+2];
         if(sar>price_low)
           {
            SaveLastReverse(i+1,true,start,price_low,last_high,ep,sar);
            start=Step; dirlong=false; ep=price_low;
            last_low=price_low;
            SarBuffer[i]=last_high;
            i--;
            continue;
           }
         if(ep<price_high) { last_high=price_high; ep=price_high; }
        }
      else
        {
         if(ep>price_low && (start+Step)<=Maximum)
           start+=Step;
         if(price_low<ma[i+1] && i==Bars-2)
           sar=price;
         if(sar<ma[i+1]) sar=ma[i+1];
         if(sar<ma[i+2]) sar=ma[i+2];
         if(sar<price_high)
           {
            SaveLastReverse(i+1,false,start,last_low,price_high,ep,sar);
            start=Step; dirlong=true; ep=price_high;
            last_high=price_high;
            SarBuffer[i]=last_low;
            i--;
            continue;
           }
         if(ep>price_low) { last_low=price_low; ep=price_low; }
        }
      SarBuffer[i]=sar;
      i--;
     }
//----
   return(0);
  }
//+------------------------------------------------------------------+


